package cn.com.qmth.examcloud.commons.util;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import javax.sql.DataSource;

import org.apache.commons.lang3.StringUtils;

import com.mchange.v2.c3p0.ComboPooledDataSource;
import com.mchange.v2.c3p0.DataSources;

import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;

/**
 * 类注释
 *
 * @author WANGWEI
 */
public class DBUtil {

	/**
	 * 属性注释
	 */
	private static final ExamCloudLog LOG = ExamCloudLogFactory.getLog(DBUtil.class);

	/**
	 * 属性注释
	 */
	private static final Object LOCK = new Object();

	/**
	 * 属性注释
	 */
	private static Map<String, DataSource> dataSources = new ConcurrentHashMap<String, DataSource>();

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param dataSourceName
	 * @return
	 */
	public static DataSource initDataSource(final String dataSourceName) {
		final ComboPooledDataSource dataSource = new ComboPooledDataSource();
		Connection conn = null;
		try {
			String dbDriver = PropertiesUtil.getString(dataSourceName + ".driver");
			String url = PropertiesUtil.getString(dataSourceName + ".url");
			String userName = PropertiesUtil.getString(dataSourceName + ".username");
			String password = PropertiesUtil.getString(dataSourceName + ".password");
			int minPoolSize = PropertiesUtil.getInt(dataSourceName + ".minPoolSize", 0);
			int maxPoolSize = PropertiesUtil.getInt(dataSourceName + ".maxPoolSize", 10);

			dataSource.setDriverClass(dbDriver);
			dataSource.setJdbcUrl(url);
			dataSource.setUser(userName);
			dataSource.setPassword(password);
			dataSource.setMinPoolSize(minPoolSize);
			dataSource.setMaxPoolSize(maxPoolSize);
			dataSource.setInitialPoolSize(minPoolSize);
			dataSource.setPreferredTestQuery("select 1 from dual");
			dataSource.setTestConnectionOnCheckin(true);
			dataSource.setAcquireRetryAttempts(1);
			dataSource.setAcquireRetryDelay(1000);
			dataSource.setMaxIdleTime(60);
			dataSource.setCheckoutTimeout(2000);
			dataSource.setAcquireIncrement(1);
			dataSource.setBreakAfterAcquireFailure(true);

			conn = dataSource.getConnection();

			Runtime.getRuntime().addShutdownHook(new Thread(new Runnable() {
				public void run() {
					try {
						DataSources.destroy(dataSource);
					} catch (SQLException e) {
						LOG.error("[JDBC] Fail to destroy dataSource. dataSourceName="
								+ dataSourceName, e);
					}
				}
			}));
		} catch (Exception e) {
			LOG.error("[JDBC] Fail to init dataSource. dataSourceName=" + dataSourceName, e);
			throw new RuntimeException(e);
		} finally {
			close(conn);
		}

		return dataSource;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param dataSourceName
	 * @return
	 */
	public static DataSource getDataSource(String dataSourceName) {
		DataSource dataSource = dataSources.get(dataSourceName);
		if (null == dataSource) {
			synchronized (LOCK) {
				dataSource = dataSources.get(dataSourceName);
				if (null == dataSource) {
					dataSource = initDataSource(dataSourceName);
					dataSources.put(dataSourceName, dataSource);
				}
			}
		}
		return dataSource;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param dataSourceName
	 * @return
	 */
	public static Connection getConnection(String dataSourceName) {
		Connection conn = null;
		try {
			conn = getDataSource(dataSourceName).getConnection();
		} catch (SQLException e) {
			LOG.error("[JDBC] Fail to get connection.", e);
		}

		return conn;
	}

	/**
	 * @param conn
	 * @param stmt
	 * @param res
	 */
	public static void close(Connection conn, Statement statement, ResultSet rs) {
		close(rs);
		close(conn, statement);
	}

	/**
	 * @param conn
	 * @param statement
	 */
	public static void close(Connection conn, Statement statement) {
		close(statement);
		close(conn);
	}

	/**
	 * @param stmt
	 * @param res
	 */
	public static void close(Statement statement, ResultSet rs) {
		close(statement);
		close(rs);
	}

	/**
	 * @param conn
	 */
	public static void close(Connection conn) {
		if (conn != null) {
			try {
				conn.close();
			} catch (SQLException e) {
				LOG.error("[JDBC] Fail to close Connection.", e);
			}
		}
	}

	/**
	 * @param statement
	 */
	public static void close(Statement statement) {
		if (statement != null) {
			try {
				statement.close();
			} catch (SQLException e) {
				LOG.error("[JDBC] Fail to close Statement.", e);
			}
		}
	}

	/**
	 * @param rs
	 */
	public static void close(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
				LOG.error("[JDBC] Fail to close ResultSet.", e);
			}
		}
	}

	/**
	 * 转换为数据库模糊查询匹配模式
	 *
	 * @author WANGWEI
	 * @param column
	 * @return
	 */
	public static String toSqlSearchPattern(String column) {
		if (StringUtils.isBlank(column)) {
			return "%";
		} else {
			column = column.trim().replaceAll("\\s", "%");
			column = "%" + column + "%";
			return column;
		}
	}

}